# File tres/Scripts/stres.cl
# November 9, 2009
# By Doug Mink, SAO Telescope Data Center

# STRES - Process multiple similar TRES sky flat images from raw data files

procedure stres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=0.95	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=no	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="sum"	{prompt="Process combined 2-D file (sum mean median only(sum) no)"}
bool	makeref=yes	{prompt="Make aperture reference(s) and flattening file from sum"}
bool	interact=yes	{prompt="Run tmakeref interactively (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=no	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath, flatfile
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot, sum
bool	vb, dt, sr, sv, disp, rc, delim, db
real	ct

	vb = verbose
	db = debug
	dt = deltemp
	sv = saveold
	sr = saveraw
	ddir = datedir
	idir = instdir
	ims = images
	delim = delimage
	ct = threshold
	sum = sumspec

	if (substr (ims,1,1) == "@")
	    fileroot (substr (ims,2,100))
	else
	    fileroot (ims)
	if (sum == "only" || sum == "sum") {
	    trssum.outroot = fileroot.rootname
	    }
	else if (sum == "mean") {
	    trsavg.outroot = fileroot.rootname
	    }
	else if (sum == "median") {
	    trsmed.outroot = fileroot.rootname
	    }

	if (ims == "") {
	    print ("STRES: Reduce multiple similar TRES flat field images, removing cosmic rays")
	    print ("Usage: ftres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (!access (ims)) {
		print ("STRES: ",ims," list file not found")
		return
		}
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles.adddate = no
	trsfiles.keeppath = no
	trsfiles.extension = ".fits"
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("STRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = no
	tproc.cosmic = yes
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, flatten=no, extract=no, disperse=no)

# Compute extraction aperture map and flattening image for average or sum
	if (makeref && sum != "no") {
	    flatfile = tproc.outfile
	    tmakeref.idbdir = "tresdata$"
	    tmakeref.idbroot = "flat$"
	    tmakeref.interact = interact
	    tmakeref.makeflat = no
	    tmakeref.split = yes
	    tmakeref.verbose = vb
	    tmakeref.debug = db
	    tmakeref (flatfile)
	    }

	if (delim) {
	    imdelete (ims)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)
end

# Mar 10 2009	New task based on FTRES
# May 12 2009	Drop avgspec parameter; add mean and median options to sumspec
# May 12 2009	Use TPROC outfile instead of parameter from combining subroutine
# Jul  2 2009	Set all parameters of TRSFILES
# Sep 30 2009	Exit with error message if list file is not found
# Nov  9 2009	Set saveraw default to no
