# File tres/Scripts/tcal.cl
# By Doug Mink, SAO Telescope Data Center
# September 28, 2009

# TCAL -- Dispersion correct list of calibration lamp spectra and write
#	  a table of images, MJD's, and shifts

procedure tcal (spectra)

string	spectra = ""	{prompt="Input multispec image(s) to correct"}

string	refroot="comp"	{prompt="Root name for comparison reference file"}
string	refdir="tresdata$" {prompt="Directory for comparison reference file"}
real	pshift=INDEF	{prompt="Pixel shift from reference file (fit if INDEF)"} 
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
file	compstd=""	{prompt="Name of file in list to use as shift standard"}
real    shift=INDEF	{prompt="Mean pixel shift (returned)"}
bool    interactive=no	{prompt="Run ecidentify on spectrum (yes or no)"}
bool	reidentify=no	{prompt="Rerun ecreidentify after xcorr (yes or no)"}
bool	redo=yes		{prompt="Rerun tcreid even if already run (yes or no)"}
bool	keepshift=no	{prompt="Keep TIDSHIFT shift if yes or cross-correlate "}
bool    disperse=yes    {prompt="Dispersion correct extracted spectra (yes or no)"}
int	ncols=65536	{prompt="Number of columns into which to rebin"}
bool	saveold=yes	{prompt="Save original file to Nodisp/ (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}
bool	xdebug=no	{prompt="Print xc debugging information (yes or no)"}

struct *calfiles

begin
	bool imfirst, ia, id, cid
	file imin, templist, imout, imref
	string ims, imroot, fib1, fib2, fib3, dline1, dline2, fname, rname
	string mjdmid, compdb
	int nshift1, nshift2, nshift3, maxlfile, maxrfile
	real tshift1, tshift2, tshift3

# If input image is a list prepend @ if not already there
	ims = spectra
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // ims
            }

	cid = interactive

	tcal1.refroot = refroot
	tcal1.refdir = refdir
	tcal1.redo = redo
	tcal1.imref = ""
	tcal1.shift = 0.0
	tcal1.pshift = pshift
	tcal1.medshift = medshift
	tcal1.disperse = disperse
	tcal1.ncols = ncols
	tcal1.saveold = saveold
	tcal1.verbose = verbose
	tcal1.debug = debug
	tcal1.xdebug = xdebug
	nshift1 = 0
	nshift2 = 0
	nshift3 = 0
	tshift1 = 0.0
	tshift2 = 0.0
	tshift3 = 0.0

#  Make temporary list of input files
        templist = mktemp ("tempcomp") // ".list"
        files (ims, >templist)
	imfirst = yes

        calfiles = templist
        while (fscan (calfiles,imin) != EOF) {
	    if (imin == "")
		break
	    if (substr (imin, 1, 1) == "#")
		next
	    smatch (imin, ".ec.")
	    if (smatch.found) {
		if (verbose)
		    print ("TCAL: Processing ", imin)
		tcal1 (imin, imref="")
		ia = interactive
		id = reidentify
		tcal1.shift = 0.0
		tcal1.imref = ""
		tcal1 (imin, imref="", reidentify=id, interactive = ia)
		imref = tcal1.imref
		shift = tcal1.shift

		# Add this shift to the shift database for this apfib
		tharadd (imin, imref, shift)

		imgetp (imin, "FIBER")
		if (imgetp.value == "1") {
		    tshift1 = tshift1 + tcal1.shift
		    nshift1 = nshift1 + 1
		    }
		else if (imgetp.value == "2") {
		    tshift2 = tshift2 + tcal1.shift
		    nshift2 = nshift2 + 1
		    }
		else {
		    tshift3 = tshift3 + tcal1.shift
		    nshift3 = nshift3 + 1
		    }
		}
	    else {
		tfib1 (imin)
		fib1 = tfib1.fib1
		fib2 = tfib1.fib2
		fib3 = tfib1.fib3
		fileroot (imin)
		imroot = fileroot.rootname
		if (fib1 == "COMP") {
		    imout = imroot // "_f1.ec.fits"
		    if (!access (imout))
			imout = imroot // "_f1.ec.fits"
		    if (verbose)
			print ("TCAL: Processing fiber 1 ", imout)
		    tcal1.shift = 0.0
		    tcal1.imref = ""
		    ia = no
		    id = no
		    if (compstd == "first" && imfirst) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    else if (imin == compstd) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    tcal1 (imout, reidentify=id, interactive = ia)
		    if (ia || id) {
			tharset (imout)
			imref = tharset.comppath
			shift = 0.0
			}
		    else {
			imref = tcal1.imref
			shift = tcal1.shift
			}

		    # Add this shift to the shift database for this apfib
		    tharadd (imout, imref, shift)
		    tshift1 = tshift1 + tcal1.shift
		    nshift1 = nshift1 + 1
		    }
		if (fib2 == "COMP") {
		    imout = imroot // "_f2.ec.fits"
		    if (!access (imout))
			imout = imroot // "_f2.ec.fits"
		    if (verbose)
			print ("TCAL: Processing fiber 2 ", imout)
		    tcal1.shift = 0.0
		    tcal1.imref = ""
		    id = no
		    ia = no
		    if (compstd == "first" && imfirst) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    else if (imin == compstd) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    tcal1 (imout, reidentify=id, interactive=ia)
		    if (ia || id) {
			tharset (imout)
			imref = tharset.comppath
			shift = 0.0
			}
		    else {
			imref = tcal1.imref
			shift = tcal1.shift
			}

		    # Add this shift to the shift database for this apfib
		    tharadd (imout, imref, shift)
		    tshift2 = tshift2 + tcal1.shift
		    nshift2 = nshift2 + 1
		    }
		if (fib3 == "COMP") {
		    imout = imroot // "_f3.ec.fits"
		    if (!access (imout))
			imout = imroot // "_f3.ec.fits"
		    if (verbose)
			print ("TCAL: Processing fiber 3", imout)
		    tcal1.shift = 0.0
		    tcal1.imref = ""
		    id = no
		    ia = no
		    if (compstd == "first" && imfirst) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    else if (imin == compstd) {
			if (cid)
			    ia = yes
			else
			    id = yes
			}
		    tcal1 (imout, reidentify=id, interactive=ia)
		    if (ia || id) {
			tharset (imout)
			imref = tharset.comppath
			shift = 0.0
			}
		    else {
			imref = tcal1.imref
			shift = tcal1.shift
			}

		    # Add this shift to the shift database for this apfib
		    tharadd (imout,imref, shift)
		    tshift3 = tshift3 + tcal1.shift
		    nshift3 = nshift3 + 1
		    if (verbose)
			print ("TCAL: Returned shift is ",tcal1.shift)
		    }
		}
	    }

# Delete temporary file list if requested
	if (deltemp) {
	    delete (templist)
	    }
	calfiles = ""

# Compute mean pixel shift
	if (verbose) {
	    if (nshift1 > 0) {
		shift = tshift1 / real (nshift1)
		print ("TCAL: fiber 1 mean shift = ",shift," from ", nshift1, " spectra")
		}
	    if (nshift2 > 0) {
		shift = tshift2 / real (nshift2)
		print ("TCAL: fiber 2 mean shift = ",shift," from ", nshift2, " spectra")
		}
	    if (nshift3 > 0) {
		shift = tshift3 / real (nshift3)
		print ("TCAL: fiber 3 mean shift = ",shift," from ", nshift3, " spectra")
		}
	    }

end

# Jan 29 2008	New task to process multiple calibration files at once
# Feb 11 2008	Change name of temporary file from tempcomp* to tempcomp*list
# Mar 11 2008	Change name of reference parameters from id* to ref*
# Mar 11 2008	Make up single fiber names if input list has image files
# Mar 11 2008	Rename task TCAL from TMCAL; call TCAL1, not TCAL
# Mar 15 2008	Add shift parameter to return mean shift
# Mar 15 2008	Add interactive parameter to make running ecidentify optional
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Aug 13 2008	Name files _f1 _f2 _f3 instead of _fib1 _fib2 _fib3
# Aug 13 2008	Use TFIB1 instead of TRSFIB1
# Aug 14 2008	Add redo to redo a night's spectra
# Aug 14 2008	Assign fib1-3 before running TCAL1 because it runs tfib1, too
# Aug 14 2008	Assign imroot before running TCAL1 because it runs fileroot, too
# Aug 15 2008	Make dispersion optional with disperse parameter
# Sep  9 2008	Add debug parameter to print TMODID output
# Sep  9 2008	Add xdebug parameter to separate pxcsao debugging
# Sep 12 2008	Add pshift parameter to set a specific pixel shift
# Sep 19 2008	Add medshift parameter to allow option of median not mean
# Sep 25 2008	Fix prompt directory for saveold
# Oct  7 2008	Check for *_fibi.ec.fits if *_fi.ec.fits not found
# Dec 30 2008	Add reidentify and ncols to match tcal1.cl
# Dec 30 2008	Add compstd and make shift table
# Dec 31 2008	Make @ optional for *.list file input

# Mar 10 2009	Debug!!
# Mar 16 2009	Process list of extracted images and set ThAr standard
# Mar 17 2009	Change default value of deltemp to yes
# Sep 21 2009	If first character of file name is #, skip it
# Sep 28 2009	Change "fib"to "f" in file names
