# File tres/Scripts/tdisp1.cl
# December 17, 2009
# By Doug Mink

#  TDISP1 -- Dispersion correct a TRES spectrum

procedure tdisp1 (spectrum)

string	spectrum=""	{prompt="Image to correct"}

string	refpath=""	{prompt="Path name for reference comparison file"}
string	refroot="comp"	{prompt="Root name for reference comparison file"}
string	refdir="tresdata$" {prompt="Directory for reference comparison file"}
string	refselect="interp" {prompt="Method (closest, interp, fit, reference, two)", enum="closest|interp|fit|reference|two"}
real	shift=0.0	{prompt="Pixel shift of reference comparison file"}
bool	saveold = yes	{prompt="Save original file to Nodisp/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print extra information (y or n)"}

begin

    string imin, calref, croot, cdir, tsize, tbin, fiber, history, rsel
    string rroot
    file imref, impix, rdir, newfile, calpath, rpath, dbpath, rfile
    file calfile1, calfile2, calfile
    bool vb, db
    real shft, calwt1, calwt2

    rdir = refdir
    rroot = refroot
    imin = spectrum
    rpath = refpath
    shft = shift
    rsel = refselect
    vb = verbose
    db = debug

# Skip this file if it has already been dispersion-corrected
    imgetp.default = "0"
    imgetp (imin, "TDISP")
    if (imgetp.value != "0") {
	if (vb) {
	    print ("TDISP1: ", imin, " has been dispersion-corrected")
	    print ("TDISP = ", imgetp.value)
	    }
	return
	}

# Find wavelength calibration file for this spectrum

# Use specified wavelength calibration file
    if (rsel == "reference") {
	tres.filename (refpath)
	rfile = tres.filename.filename
	rpath = refpath
	fileroot (rfile)
	calref = fileroot.rootname
	calpath = "databaseec/ec" // calref
	if (!access (calpath)) {
	    dbpath = rpath // "database/ec" // calref
	    copy (dbpath, calpath)
	    }
	}

# Interpolate dispersion-direction pixel shifts from nearby COMP spectra
    else if (rsel == "closest" || rsel == "interp" || rsel == "fit") {
	tharshift1 (spectrum, refroot=rroot, refselect=rsel, verbose=vb, debug=db)
	rfile = tharshift1.refpath
	shft = tharshift1.shift
	if (shft == 0.) {
	    tdispref (imin, refroot=refroot, refdir=rdir, verbose=vb, debug=db)
	    imref = tdispref.reffile
	    rpath = tdispref.refpath
	    }
	else {
	    rpath = tharshift1.refpath
	    tres.filename (rpath)
	    imref = tres.filename.filename
	    rdir = tres.filename.dirpath
	    shft = tharshift1.shift
	    imgetp.default = rpath
	    imgetp (rpath, "COMPFILE")
	    refpath = imgetp.value
	    imgetp.default = ""
	    }

# Copy reference database/id file with shift from correlation
	tmodid.indir = rdir
	tmodid.outdir = ""
	tmodid.shift = shft
	tmodid.keepfit = yes
	tmodid.scale = 1.0
	tmodid.verbose = vb
	tmodid.debug = db
	tmodid (imref, imin)
	ecrefit (imin)
	fileroot (imin)
	calref = fileroot.rootname
	calpath = "database/ec" // calref
	}

# Interpolate line positions from closest COMP spectra in both directions
    else if (rsel == "two") {
	tharshift1 (spectrum, refselect=rsel, verbose=vb, debug=db)
	fileroot (tharshift1.refspec1)
	calfile1 = "ec" // fileroot.rootname
	calwt1 = tharshift1.refwt1
	fileroot (tharshift1.refspec2)
	calfile2 = "ec" // fileroot.rootname
	calwt2 = tharshift1.refwt2
	calfile = "ec" // calref
	calpath = "database/ec" // calref
	twoid.indir = ""
	twoid.outdir = ""
	twoid.keepfit = yes
	twoid (calfile1, calwt1, calfile2, calwt2, calfile)
	}

    else {
	print ("TDISP1: Mode ",rsel," is not supported")
	return
	}

# If id file for this file does not exist, exit
    if (!access (calpath)) {
	print ("TDISP1: ", calpath, " does not exist")
	return
	}

    if (shft != 0.0) {
	hedit (imin, "TIDSHIFT", (shft), add+, update+,verify-)
	hedit (imin, "TIDREF", rpath, add+, update+,verify-)
	}

# Use local copy of shifted reference spectrum for dispersion
    hedit (imin, "REFSPEC1", calref, add+, update+,verify-)

# Set dispersion task parameters
    dispcor.linearize = no
    dispcor.database = "database"
    dispcor.table = ""
    dispcor.w1 = INDEF
    dispcor.w2 = INDEF
    dispcor.dw = INDEF
    dispcor.nw = INDEF
    dispcor.log = no
    dispcor.flux = yes
    dispcor.blank = 0.
    dispcor.samedisp = no
    dispcor.global = no
    dispcor.ignoreaps = no
    dispcor.confirm = no
    dispcor.listonly = no
    dispcor.verbose = vb
    dispcor.logfile = ""

# Add dispersion function to spectrum file header
    fileroot (calref)
    impix = fileroot.rootname // "p.ec.fits"
    rename (imin, impix)
    dispcor (impix, imin, linearize=no)
    if (!access (imin)) {
	print ("TDISP1: DISPCOR failed on ",imin)
	rename (impix, imin)
	return
	}

# Add history line including time
    gettime (0.0, timediff=99)
    history = gettime.date // " " // gettime.time
    history = history // " dispersion fit from " // refpath
    hedit (imin, "TDISP", history, add+, verify-)

# Move original file to Nodisp directory
    if (saveold) {
	if (!access ("Nodisp/")) {
	    mkdir ("Nodisp")
	    if (vb)
		print ("TDISP1: Making directory Nodisp/")
	    }
	newfile = "Nodisp/" // imin
	if (access (newfile)) {
	    imdelete (newfile)
	    }
	rename (impix, newfile)
	if (vb) {
	    print ("TDISP1: Original ", imin, " => Nodisp/")
	    }
	}
    else {
	delete (impix)
	}

end

# Jan 28 2008	New task
# Jan 29 2008	Add saveold to save undispersed spectrum to Nodisp/
# Jan 29 2008	Write TCAL FITS parameter to output file
# Jan 30 2008	Note that directory is being created if verbose
# Feb 13 2008	Return immediately if file is already dispersion-corrected
# Feb 25 2008	Rename from TDISP to TDISP1 since it only operates on one file
# Mar 11 2008	Exit if no reference root is specified and id file is missing
# Mar 11 2008	If shift is not zero, write it to header as TIDSHIFT
# Mar 18 2008	Add missing quotation mark
# Apr 29 2008	Delete undispersed file if not saving it
# May 16 2008	Add input time argument to gettime() 
# May 23 2008	Add refpath parameter to specify reference spectrum
# May 23 2008	Interpolate between closest ThAr spectrum shifts if refpath=today
# Jul 18 2008	Assume that reference spectrum is in current working directory
# Jul 21 2008	Pass refdir and refroot to tharshift1()
# Jul 28 2008	Keep shifted fit from calibration spectrum
# Aug 15 2008	Use ECREFIT to fit shifted ThAr lines for dispersion function
# Sep 19 2008	Fix bug and call ECREFIT instead of non-existen TREFIT
# Nov 13 2008	Specify task tres.filename
# Dec 15 2008	Add timediff argument to GETTIME to get current time

# Dec 17 2009	Add "two" option to combine line centers from two ThAr spectra
