# TEXTRACT - Extract spectra from TRES images and dispersion-correct
# September 1, 2009
# By Doug Mink, SAO Telescope Data Center

procedure textract (images)

string	images=""	{prompt="Images from which to extract spectra"}
bool	disperse=yes	{prompt="Add wavelength dispersion function to spectrum (yes or no)"}
string	aproot="flat"	{prompt="Root filename for aperature reference spectrum"}
string	apdir="tresdata$" {prompt="Directory for reference spectrum"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	refroot="comp"	{prompt="Root filename for wavelength reference spectrum"}
string	refdir="tresdata$" {prompt="Directory for reference spectrum"}
string	refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
bool	rmblaze=no	{prompt="Remove echelle blaze function (yes or no)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
file	compstd=""	{prompt="File to use as ThAr shift standard (first sum mean all"}
bool	compid=yes	{prompt="Interactively ecidentify standard if yes; ecreid if no"}
bool    subsky=no       {prompt="Subtract sky spectrum (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *extfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	istd, cstd, newtemp, dfile, tpfile
file	imfile, newfile, imlist, tref1, tref2, compdb, imin1, imin2, imin3
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex, rname
string	binning, temparg, apd, apr, mjdmid, fname
string	object, imref, imlast1, imlast2, imlast3
bool	vb, dt, sv, fp, fx, fb, flat, disp, recent, svr, sum, mshift, bov
bool	mean, ia, id, imfirst, cid, rd, rmb, rms, dsub, subs, median, only
bool	bsub
int	nfiles, maxlfile, maxrfile
real	tshift1, tshift2, shift

	vb = verbose
	dt = deltemp
	sv = saveold
	rd = yes
	subs = subsky
	recent = recenter
	disp = disperse
	apr = aproot
	apd = apdir
	mshift = medshift
	imx = ".fits"
	cid = compid
	cstd = compstd
	rmb = rmblaze

	imlast1 = ""
	imlast2 = ""
	imlast3 = ""

	ims = image
	if (ims == "") {
	    print ("TEXTRACT: Process TRES image files from extraction")
	    print ("Usage: textract im1,im2,...,imn")
	    print ("Usage: textract *.fits")
	    print ("Usage: textract @image_list_file")
	    return
	    }

# If input image is a list prepend @ if not already there
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // images
            }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempproc") // ".list"
	files (ims, >templist)

# Extract 1D x orders spectrum and add dispersion, if requested
	extfiles = templist
	tshift1 = 0.0
	tshift2 = 0.0
	imfirst = yes
        while (fscan (extfiles,imin) != EOF) {

# Extract however many spectra are in this image
	    textract1 (imin, aproot=apr, apdir=apd, redo=rd, recenter=recent, verbose=vb)

# Add wavelength dispersion, if requested
	    if (disp) {
		tfib1 (imin)
		object = tfib1.object
		imin1 = textract1.spec1
		imin2 = textract1.spec2
		imin3 = textract1.spec3

# Make throughput correction spectrum if it will be needed
		if (subs) {
		    flat1 = aproot // tfib1.apsize // tfib1.binning // "1.ec.fits"
		    flat2 = aproot // tfib1.apsize // tfib1.binning // "2.ec.fits"
		    tpfile = tpmake.tproot // tfib1.apsize // tfib1.binning // "12.ec.fits"
		    if (!access (tpfile))
			    tpmake (flat1, flat2, reuse=yes)
		    }

# Add wavelength dispersion to fiber 3 spectrum
		if (imin3 != "") {
		    if (rmb) {
			tblaze1 (imin3, saveold=sv, verbose=vb)
			}
		    imgetp.default = "NONE"
		    imgetp (imin3, "FIB3")
		    if (imgetp.value == "COMP") {
			tcal1.shift = 0.0
			tcal1.saveold = sv
			tcal1.deltemp = dt
			tcal1.verbose = vb
			tcal1.redo = rd
			ia = no
			id = no
			istd = ""
			if (cstd == "first") {
			    if (imfirst) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    }
			else if (cstd == "all") {
			    if (!imfirst) {
				istd = imlast3
				}
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else if (imin == cstd) {
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else {
			    istd = cstd
			    }
			tcal1 (imin3, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			if (ia || id) {
			    tharset (imin3)
			    imref = tharset.comppath
			    shift = 0.0
			    }
			else {
			    imref = tcal1.imref
			    shift = tcal1.shift
			    }
			tshift1 = shift
			tshift2 = shift
			print ("TPROC: simultaneous ThAr not working yet")

			# Add this shift to the shift database for this apfib
			tharadd (imin3, imref, shift)
			imlast3 = imin3
			}
		    else 
			tdisp1 (imin3, refselect=refmode, refroot=refroot, refdir=refdir, saveold=sv, verbose=vb)
		    imlast3 = imin3
		    }

# Compute pixel shifts for apertures 1 and 2 from existing ThAr spectra
		else if (tfib1.imagetype != "COMP") {
		    if (imin1 != "") {
			tharshift1 (imin1, refroot=refroot, refselect=refmode, verbose=vb)
			tref1 = tharshift1.refpath
			tshift1 = tharshift1.shift
			}
		    if (imin2 != "") {
			tharshift1 (imin2, refroot=refroot, refselect=refmode, verbose=vb)
			tref2 = tharshift1.refpath
			tshift2 = tharshift1.shift
			}
		    }

# Add wavelength dispersion to fiber 1 spectrum
		if (imin1 != "") {
		    if (rmb) {
			tblaze1 (imin1, saveold=sv, verbose=vb)
			}
		    imgetp.default = "NONE"
		    imgetp (imin1, "FIB1")

# Compute shift and fit dispersion for ThAr spectrum
		    if (imgetp.value == "COMP") {
			tcal1.shift = 0.0
			tcal1.saveold = sv
			tcal1.deltemp = dt
			tcal1.verbose = vb
			tcal1.redo = rd
			ia = no
			id = no
			istd = ""
			if (cstd == "first") {
			    if (imfirst) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    }
			else if (cstd == "all") {
			    if (!imfirst) {
				istd = imlast1
				}
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else if (imin == cstd) {
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else {
			    istd = cstd
			    }
			tcal1 (imin1, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			if (ia || id) {
			    tharset (imin1)
			    imref = tharset.comppath
			    shift = 0.0
			    }
			else {
			    imref = tcal1.imref
			    shift = tcal1.shift
			    }
			tshift1 = shift

			# Add this shift to the shift database for this apfib
			tharadd (imin1, imref, shift)
			imlast1 = imin1
			}
#		    else if (imin3 != "")
#			tdisp1 (imin1, shift=tshift1, refpath=tref1, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
		    else
			tdisp1 (imin1, refpath="today", refselect=refmode, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
		    }

# Add wavelength dispersion to fiber 2 spectrum
		if (imin2 != "") {
		    if (rmb) {
			tblaze1 (imin2, saveold=sv, verbose=vb)
			}
		    imgetp.default = "NONE"
		    imgetp (imin2, "FIB2")

		# Compute shift and fit dispersion for ThAr spectrum
		    if (imgetp.value == "COMP") {
			tcal1.shift = 0.0
			tcal1.saveold = sv
			tcal1.deltemp = dt
			tcal1.verbose = vb
			tcal1.redo = rd
			id = no
			ia = no
			istd = ""
			if (cstd == "first") {
			    if (imfirst) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    }
			else if (cstd == "all") {
			    if (!imfirst) {
				istd = imlast2
				}
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else if (imin == cstd) {
			    if (cid)
				ia = yes
			    else
				id = yes
			    }
			else {
			    istd = cstd
			    }
			tcal1 (imin2, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			if (ia || id) {
			    tharset (imin2)
			    imref = tharset.comppath
			    istd = imref
			    shift = 0.0
			    }
			else {
			    imref = tcal1.imref
			    shift = tcal1.shift
			    istd = imref
			    }
			tshift2 = shift

			# Add this shift to the shift database for this apfib
			tharadd (imin2, imref, shift)
			imlast2 = imin2
			}
#		    else if (imin3 != "")
#			tdisp1 (imin2, shift=tshift2, refpath=tref2, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
		    else
			tdisp1 (imin2, refpath="today", refselect=refmode, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
		    }
		}
	    imfirst = no
	    }

	if (dt)
	    delete (templist)
	extfiles = ""
end

# Sep  1 2009	New program including last half of TPROC
