# File tres/Scripts/tmakemask.cl
# By Doug Mink
# April 17, 2009

# TMAKEMASK -- Make a mask file to remove the spectrum from a TRES image
#          so the background can be fit with a surface

procedure tmakemask (infile)

file	infile=""	{prompt="Flattening file from which to make a mask file"}

string	maskroot="flat"	{prompt="Root file name for output mask file"}
bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print output (yes or no)"}

begin

    bool vb, db
    file ifile, outfile, filepath
    string apfib, datedir

    vb = verbose
    db = debug

    tcheck

# Quit if input flattening file is not there
    ifile = infile
    if (!access (ifile)) {
	print ("TMAKEMASK: Flattening file ",ifile, " does not exist")
	return
	}

# Find fiber information for this file
    tfib1 (ifile, specdir="")
    apfib = tfib1.apfib

    tcheck
    if (tcheck.datedir == "") {
	print ("TMAKEMASK: No date directory set")
	return
	}
    datedir = tcheck.datedir

# Delete existing output mask field file if already exists
    outfile = maskroot // apfib // ".mask.fits"
    if (access (outfile)) {
	if (vb) {
	    print ("TMAKEMASK: Deleting existing ", outfile)
	    }
	delete (outfile)
	}

# Make a mask file with zero where there is spectrum, one everywhere else
    tmaskfile (infile, outfile, datedir, verbose=vb, debug=db)

    if (vb) {
	printf ("TMAKEMASK: Making mask file %s from %s\n", outfile, ifile)
	}
end

# Apr 17 2009	New task making file to mask out spectra from flattening file
