# File tres/Scripts/tmakeref.cl
# By Doug Mink
# October 9, 2009

# TMAKEREF -- Get standard aperture reference (=flat) file for flat field file
#             from database, run aptrace to update database ap* file, make new
#             apflattening file, flatten this file, and split database file if
#             there are two fibers in it.  Then make two-aperture mask (or 1
#             if fiber 3), remove background from flat image(s), extract
#             spectra, and make throughput correction spectrum and normalize
#             and smooth into blaze removal spectra

procedure tmakeref (flatfile)

file flatfile=""	{prompt="Flat image file to make default"}

string  idbdir="tresdata$"	{prompt="Directory for standard flat file"}
string  idbroot="flat"	{prompt="Root file name for standard flat file"}
int	aporder=5	{prompt="Initial order of aptrace fit to each order"}
bool	interact=yes	{prompt="Run aptrace interactively (yes or no)"}
bool	makeflat=yes	{prompt="Run apflatten on resulting flat file (yes or no)"}
bool	split=yes	{prompt="Split fibers 1 and 2 for extraction (y or n)"}
bool    rmscat=yes	{prompt="Remove scattered light from image (yes or no)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print output (yes or no)"}

begin

    bool vb, db, first, first1, first2, odd, bfit, ia, sv, dt, rms
    int i, io, ncoeff, icoeff, nfib
    file idbpath, odbpath, odbpath1, odbpath2, spec1, spec2, spec3, normfile
    file ffile, outfile, outfile1, outfile2, oldfile, normfile2, newnorm
    string flat, idbfile
    string outroot, outroot1, outroot2, filepath, dataroot, apfib

    vb = verbose
    db = debug
    flat = "flat"
    ia = interact
    sv = saveold
    dt = deltemp
    rms = rmscat

    tcheck

# Quit if input flat field file is not there
    ffile = flatfile
    if (!access (ffile)) {
	print ("TMAKEREF: Flat field file ",ffile, " does not exist")
	return
	}

# Find fiber information for this file
    tfib1 (ffile)
    apfib = tfib1.apfib

# Quit if input database entry does not exist
    idbfile = "ap" // idbroot // apfib
    idbpath = idbdir // "database/" // idbfile
    if (!access (idbpath)) {
	printf ("TMAKEREF: Database file %s does not exist\n", idbpath)
	return
	}

# Delete existing output database entry if already exists
    dataroot = "database/ap"
    odbpath = "database/" // idbfile
    if (!access ("database"))
	mkdir ("database")
    if (access (odbpath)) {
	if (vb) {
	    print ("TMAKEREF: Deleting existing ", odbpath)
	    }
	delete (odbpath)
	}

# Copy standard aperture solution to current database
    copy (idbpath, odbpath)

# Delete existing output standard flat field file if already exists
    outfile = flat // apfib // ".fits"
    if (access (outfile)) {
	if (vb) {
	    print ("TMAKEREF: Deleting existing ", outfile)
	    }
	delete (outfile)
	}

# Copy input flat field file into standard-named flat field file
    copy (ffile, outfile)
    gettime (tfib1.mjd)
    filepath = gettime.datadir // "/" // ffile
    hedit (outfile, "FLATFILE", filepath, add+, update+, verify-, show=vb)

# Retrace all 51 or 102 apertures in output file
    if (tfib1.fibkey == "FF")
	aptrace.apertures = "1-102"
    else
	aptrace.apertures = "1-51"
    if (ia)
	aptrace.references = ""
    else
	aptrace.references = "OLD"
    aptrace.interactive = ia
    aptrace.find = yes
    aptrace.recenter = yes
    aptrace.resize = yes
    aptrace.edit = ia
    aptrace.trace = yes
    aptrace.fittrace = ia
    aptrace.line = INDEF
    aptrace.nsum = 10
    aptrace.step = 10
    aptrace.nlost = 3
    aptrace.function = "chebyshev"
    aptrace.order = 5
    aptrace.sample = "*"
    aptrace.naverage = 1
    aptrace.niterate = 0
    aptrace.low_reject = 3.
    aptrace.high_reject = 3.
    aptrace.grow = 0.
    aptrace (outfile)

# Make pixel flattening file with TMAKEFLAT (APFLATTEN) if requested
    if (makeflat) {
	normfile = flat // apfib // ".flat.fits"
	if (access (normfile))
	    if (vb) {
		print ("TMAKEREF: Deleting existing flattening file ",normfile)
	    delete (normfile)
	    }
	tmakeflat (outfile)
	oldfile = flat // apfib // "u.fits"
	if (access (oldfile))
	    delete (oldfile)

# Set up apflatten file name
	copy (outfile, oldfile)
	normfile = flat // apfib // ".flat.fits"
	tflat1.normfile = normfile

# Flatten this file with TFLAT
	if (access (normfile)) {
	    tflat1 (outfile, saveold=sv, verbose=vb)

# Re-run aptrace on flattened file
	    aptrace (outfile, interact=ia)
	    }
	}

# Split ap* database entry if there are two fibers in image
    if (tfib1.fiber == "12" && !split) {
	tsplitdb (outfile)
	}

# Combine single fiber flattening files if possible
    if (tfib1.fiber == "1" || tfib1.fiber == "2") {
	if (tfib1.fiber == "1") {
	    normfile2 = flat // tfib1.apsize // tfib1.binning // "2.flat.fits"
	    if (!access (normfile2)) {
		if (vb)
		    print ("TMAKEREF: Cannot read file ",normfile2)
		return
		}
	    }
	else if (tfib1.fiber == "2") {
	    normfile2 = flat // tfib1.apsize // tfib1.binning // "1.flat.fits"
	    if (!access (normfile2)) {
		if (vb)
		    print ("TMAKEREF: Cannot read file ",normfile2)
		return
		}
	    }
	newnorm = flat // tfib1.apsize // tfib1.binning // "12.flat.fits"
	if (access (newnorm)) {
	    if (vb)
		print ("TMAKEREF: Deleting existing flattening file ",newnorm)
	    delete (newnorm)
	    }
	imarith (normfile, "*", normfile2, newnorm)
	hedit (newnorm, "FIBKEY","FF")
	}
    else
	newnorm = normfile

# Make a mask file to remove spectrum from a TRES image
    tmakemask (newnorm)

# Remove background from flat image(s)
    if (rms) {
	tscat1 (outfile, saveold=sv, deltemp=dt)
	}
    if (tfib1.fiber == "1") {
	outfile2 = flat // tfib1.apsize // tfib1.binning // "2.fits"
	if (!access (outfile2)) {
	    if (vb)
		print ("TMAKEREF: Cannot read file ",outfile2)
	    return
	    }
	if (rms) {
	    tscat1 (outfile2, saveold=sv, deltemp=dt)
	    }
	}
    else if (tfib1.fiber == "2") {
	outfile2 = flat // tfib1.apsize // tfib1.binning // "1.fits"
	if (!access (outfile2)) {
	    if (vb)
		print ("TMAKEREF: Cannot read file ",outfile2)
	    return
	    }
	if (rms) {
	    tscat1 (outfile2, saveold=sv, deltemp=dt)
	    }
	}
    else
	outfile2 = ""

# Extract spectra from flat image(s)
    textract1 (outfile)

    spec1 = flat // tfib1.apsize // tfib1.binning // "1.ec.fits"
    if (textract1.spec1 != "") {
	if (access (spec1)) {
	    delete (spec1)
	    if (vb)
		print ("TMAKEREF: Deleting previously existing ", spec1)
	    }
	imrename (textract1.spec1, spec1)
	}
    spec2 = flat // tfib1.apsize // tfib1.binning // "2.ec.fits"
    if (textract1.spec2 != "") {
	if (access (spec2)) {
	    delete (spec2)
	    if (vb)
		print ("TMAKEREF: Deleting previously existing ", spec2)
	    }
	imrename (textract1.spec2, spec2)
	}
    spec3 = flat // tfib1.apsize // tfib1.binning // "3.ec.fits"
    if (textract1.spec3 != "") {
	if (access (spec3)) {
	    delete (spec3)
	    if (vb)
		print ("TMAKEREF: Deleting previously existing ", spec3)
	    }
	imrename (textract1.spec3, spec3)
	}
    if (outfile2 != "" && !access (spec1)) {
	textract1 (outfile2)
	if (textract1.spec1 != "")
	    imrename (textract1.spec1, spec1)
	}
    if (outfile2 != "" && !access (spec2)) {
	textract1 (outfile2)
	if (textract1.spec2 != "")
	    imrename (textract1.spec2, spec2)
	}

# Normalize extracted spectra for blaze correction
    if (access (spec1))
	tmakeblaze (spec1)
    if (access (spec2))
	tmakeblaze (spec2)
    if (access (spec3))
	tmakeblaze (spec3)
end

# Feb  6 2009	New task to copy and refit aptrace database information

# Feb 27 2009	Make split=yes default
# Mar 12 2009	If database/ directory does not exist, create it
# Mar 12 2009	Do not aplatten if apflatten file was not created
# Apr 23 2009	Move aperture database splitting to TSPLITDB task
# Apr 23 2009	Combine single fiber flattening files into two-fiber file
# Apr 23 2009	Extract flat spectra if possible and make blaze correcting files
# Apr 28 2009	Normalize flat spectra for blaze correction
# Apr 29 2009	Rename echelle spectrum files so fiber number is there only once
# May 19 2009	Add saveold and deltemp parameters and pass to TSCAT1 and TFLAT1
# Sep 19 2009	Change default interact to yes
# Sep 19 2009	Add rmscat parameter to remove scattered light after mask is made
