# File tres/Scripts/tsplitdb.cl
# By Doug Mink
# April 23, 2009

# TSPLITDB -- Make separate fiber data base entries from one apflat12 file

procedure tsplitdb (outfile)

file outfile=""	{prompt="Flat image file to make default"}

bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print output (yes or no)"}

begin

    bool vb, db, first, first1, first2, odd, bfit
    int i, ncoeff, icoeff, nfib
    file idbpath, odbpath, odbpath1, odbpath2
    file ffile, outfile1, outfile2, oldfile
    string s1, s2, s3, s4, s5, s6, s7, s8, null, flat, idbfile
    string outroot, outroot1, outroot2, dataroot, apfib
    int oldap, newap

    vb = verbose
    db = debug
    null = ""
    flat = "flat"
    dataroot = "database/ap"

    tcheck

# Quit if input flat field file is not there
    ffile = outfile
    if (!access (ffile)) {
	print ("TSPLITDB: Flat field file ",ffile, " does not exist")
	return
	}

# Find fiber information for this file
    tfib1 (ffile)
    apfib = tfib1.apfib
    if (tfib1.fiber != "12") {
	print ("TSPLITDB: Only fiber ",tfib1.fiber," in ",ffile)
	return
	}

# Quit if input database entry does not exist
    idbpath = dataroot // flat // apfib
    if (!access (idbpath)) {
	printf ("TSPLITDB: Database file %s does not exist\n", idbpath)
	return
	}
# Set output database paths
    outroot1 = flat // tfib1.apsize // tfib1.binning // "1"
    outroot2 = flat // tfib1.apsize // tfib1.binning // "2"
    odbpath1 = dataroot // outroot1
    odbpath2 = dataroot // outroot2
    nfib = 2

#    Symbolically link to new file
    outfile1 = outroot1 // ".fits"
    lns (outfile, outfile1)
    outfile2 = outroot2 // ".fits"
    lns (outfile, outfile2)

# If output database entries already exist, delete them
    if (access (odbpath1)) {
	if (vb)
	    print ("TSPLITDB: Deleting existing ", odbpath1)
	delete (odbpath1)
	}
    if (access (odbpath2)) {
	if (vb)
	    print ("TSPLITDB: Deleting existing ", odbpath2)
	delete (odbpath2)
	}

    if (vb) {
	if (nfib == 1)
	    printf ("TSPLITDB: Copying %s to %s\n",
		    idbpath, odbpath1)
	else
	    printf ("TSPLITDB: Splitting %s to %s and %s\n",
		    idbpath, odbpath1, odbpath2)
	}
    s1 = null
    s2 = null
    s3 = null
    s4 = null
    s5 = null
    s6 = null
    s7 = null
    s8 = null
    ncoeff = 0
    gettime (0.0, timediff=99, usesec+)

# Read through database entry one line at a time
# Split it in two if nfib=2 and split=yes
    first1 = yes
    first2 = yes
    odd = no
    bfit = no
    list = idbpath
    while (fscan (list, s1, s2, s3, s4, s5, s6, s7, s8) != EOF) {

# Comments lines start each aperture with date and time
	if (s1 == "#") {
	    if (odd && nfib == 2)
		odd = no
	    else
		odd = yes
	    if (odd) {
		outroot = outroot1
		odbpath = odbpath1
		first = first1
		}
	    else {
		outroot = outroot2
		odbpath = odbpath2
		first = first2
	    }
	    if (first)
		printf ("# %s %s %s\n", s2, s3, s4, >>odbpath)
#			gettime.dweek, gettime.time, gettime.date, >odbpath)
	    else
		printf ("# %s %s %s\n", s2, s3, s4, >>odbpath)
#			gettime.dweek, gettime.time, gettime.date, >>odbpath)
	    if (odd)
		first1 = no
	    else
		first2 = no

	    if (db) {
		printf ("# %s %s %s\n",
			gettime.dweek, gettime.date, gettime.time)
		}
	    }

	else if (s1 == "begin") {
	    printf ("%s\t%s %s %s %s %s\n", s1, s2, outroot, s4, s5, s6, >>odbpath)
	    if (db) {
		printf ("%s\t%s %s %s %s %s\n", s1, s2, s3, s4, s5, s6)
		}
	    }

	else if (s1 == "background") {
	    printf ("\t%s\n", s1, >>odbpath)
	    bfit = yes
	    if (db) {
		printf ("\t%s\n", s1)
		}
	    }
	else if (s1 == "axis") {
	    printf ("\t%s\t%s\n", s1, s2, >>odbpath)
	    bfit = no
	    if (db) {
		printf ("\t%s\t%s\n", s1, s2)
		}
	    }

# Pass through blank lines
	else if (s1 == null) {
	    printf ("\n", >>odbpath)
	    if (db) {
		printf ("\n")
		}
	    }

# One-token lines are fit coefficients
	else if (s2 == null) {
	    printf ("\t\t%s\n", s1, >>odbpath)
	    if (db) {
		printf ("\t\t%s\n", s1)
		}
	    }

# Two-token lines are parameter assignments
	else if (s3 == null) {
	    if (s1 == "image") {
		printf ("\timage\t%s\n", outroot, >>odbpath)
		if (db) {
		    printf ("\timage\t%s\n", outroot)
		    }
		}
	    else if (s1 == "aperture" || s1 == "beam") {
		oldap = int (s2)
		if (nfib ==2)
		    newap = ((oldap + 1) / 2)
		else
		   newap = oldap
		printf ("\t%s\t%d\n", s1, newap, >>odbpath)
		if (db) {
		    printf ("\t%s\t%d\n", s1, newap)
		    }
		}
	    else if (bfit) {
		printf ("\t\t%s %s\n", s1, s2, >>odbpath)
		if (db) {
		    printf ("\t\t%s %s\n", s1, s2)
		    }
		}
	    else {
		printf ("\t%s\t%s\n", s1, s2, >>odbpath)
		if (db) {
		    printf ("\t%s\t%s\n", s1, s2)
		    }
		}
	    }

# Three-token lines are parameter assignments, too
	else if (s4 == null) {
	    if (s1 == "image") {
		printf ("\timage\t%s\n", outroot, >>odbpath)
		if (db) {
		    printf ("\timage\t%s\n", outroot)
		    }
		}
	    else {
		printf ("\t%s\t%s %s\n", s1, s2, s3, >>odbpath)
		if (db) {
		    printf ("\t%s\t%s %s\n", s1, s2, s3)
		    }
		}
	    }

	s1 = null
	s2 = null
	s3 = null
	s4 = null
	s5 = null
	s6 = null
	s7 = null
	s8 = null
	}
end

# Apr 23 2009	New task split off from TMAKEREF
