# File tres/Src/fitspec.x
# February 10, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After RVSAO ICSUBCON subroutine

include <pkg/gtools.h>
include	<error.h>


# FITSPEC - Fit a spectrum with a function and return the fit as a spectrum

procedure fitspec (n, fdata, wltemp, name, nsm)

int	n			# Number of pixels to fit
real	fdata[ARB]		# Spectrum to be fit
real	wltemp[ARB]		# Wavelengths for spectrum
char	name[ARB]		# Name of spectrum
int	nsm			# Number of times to smooth spectrum

int	i,newrej,nrej,ns,nsp
char	title_line[SZ_LINE]
char	wlab[SZ_LINE]
real	cveval()
pointer	spec0
pointer	ic			# ICFIT ptr
pointer	gt			# GTOOLS pointer
pointer	gp			# graphics pointer
pointer	cv			# pointer for curve fitting
pointer fit			# Spectrum containing values from fit
pointer	sp
pointer	x, wts, rejpts		# buffers for curve fitting
pointer gopen(), gt_init()
errchk  gopen, gt_init

include	"tresop.com"

begin
	call strcpy ("Wavelength in Angstroms", wlab, SZ_LINE)

# Check to see if spectrum is all zeroes
	nsp = 0
	do i = 1, n {
	    if (fdata[i] != 0.0)
		nsp = nsp + 1
	    }

# If spectrum is all zeroes, return it unchanged
	if (nsp < 1) {
	    return
	    }

# Set the ICFIT pointer structure.
	call ic_open (ic)
	call ic_pstr (ic, "sample", sample)
	call ic_pstr (ic, "function", fitfunc)
	call ic_puti (ic, "naverage", naverage)
	call ic_puti (ic, "order", fitorder)
	call ic_puti (ic, "niterate", niterate)
	call ic_putr (ic, "low", lowrej)
	call ic_putr (ic, "high", hirej)
	call ic_putr (ic, "grow", grow)
	call ic_pstr (ic, "ylabel", "")

#  Allocate memory for curve fitting.
	call smark (sp)
	call salloc (x, n, TY_REAL)
	call salloc (wts, n, TY_REAL)
	call salloc (rejpts, n, TY_INT)
	call salloc (fit, n, TY_INT)

#  Initialize weighting array to all ones
	call amovkr (1., Memr[wts], n)

#  Avoid fitting zeroes at ends of spectrum by weighting them as 0
	do i = 1, n {
	    if (fdata[i] == 0.) {
		Memr[wts+i-1] = 0.
#		if (debug) {
#		    call printf ("ICSUBCON: %d = 0\n")
#			call pargi (i)
#		    call flush (STDOUT)
#		    }
		}
	    else
		break
	    }
	do i = n, 1, -1 {
	    if (fdata[i] == 0.) {
		Memr[wts+i-1] = 0.
#		if (debug) {
#		    call printf ("ICSUBCON: %d = 0\n")
#			call pargi (i)
#		    call flush (STDOUT)
#		    }
		}
	    else
		break
	    }
	call amovki (NO, Memi[rejpts], n)

#  Initlialize X array
	do i = 1, n {
	    Memr[x+i-1] = real (i)
#	    if (debug) {
#		call printf ("ICSUBCON: %d=%f: %f (%f)\n")
#		    call pargi (i)
#		    call pargr (Memr[x+i-1])
#		    call pargr (fdata[i])
#		    call pargr (Memr[wts+i-1])
#		call flush (STDOUT)
#		}
	    }

#  Update icfit struct
	call ic_putr (ic, "xmin", 1.)
	call ic_putr (ic, "xmax", real(n))

	if (debug) {
	    call printf ("ICSUBCON: %d points being fit from spectrum %s\n")
		call pargi (n)
		call pargstr (name)
	    call flush (STDOUT)
	    }

#  If the interactive flag is set then use icg_fit to set the fitting
#  parameters.  Only done if task is run interactively as well.

	if (fitint) {
	    iferr {
		gp = gopen ("stdgraph", NEW_FILE, STDGRAPH)
		gt = gt_init ()
		call gt_sets (gt, GTTYPE, "line")
		} then
		    call error (0, "Error opening `stdgraph'.")

	    call icg_fit (ic,gp,"cursor",gt,cv,Memr[x],fdata,Memr[wts],n)
	
	# Now recover any parameters that were changed
	    call recover_icfit_pars (ic)
	    call gt_free (gt)
	    call gclose (gp)
	    }

#  Do the fit non-interactively using ic_fit.
	else {
	    call ic_fit (ic,cv,Memr[x],fdata,Memr[wts],n,YES,YES,YES,YES)
	    }

#  Save original spectrum if plotting and chopping emission lines
	spec0 = NULL
	if (pltfit && choplines) {
	    call salloc (spec0,n,TY_REAL)
	    do i = 1, n {
		Memr[spec0+i-1] = fdata[i]
		}
	    }

#  Find deviant points
	if (choplines && (abrej > 0. || emrej > 0.)) {
	    nrej = 0
	    newrej = 0
	    call ic_deviantr (cv, Memr[x],fdata,Memr[wts], Memi[rejpts],
			      n,abrej,emrej,grow,NO,
			      nrej,newrej)

#  Replace points out of bounds with fit
	    do i = 1, n {
		if (Memi[rejpts+i-1] == YES)
		    fdata[i] = cveval (cv, Memr[x+i-1])
		}

# If plot enabled, plot the lines that were removed
	    if (pltfit && newrej > 0) {
		do i = 1, n {
		    Memr[spec0+i-1] = Memr[spec0+i-1] - fdata[i]
		    }
		call strcpy (name, title_line, SZ_LINE)
		if (emrej > abrej)
		    call strcat (" absorption lines", title_line, SZ_LINE)
		else 
		    call strcat (" emission lines", title_line, SZ_LINE)
		ns = nsm
		call tresplot (n,Memr[spec0],title_line,wltemp, wlab, ns, intres, ymin, ymax)
		}
	    }

#  Compute a fit to the spectrum
	if (outfit) {
	    call cvvector (cv, Memr[x], Memr[fit], n)

# Return fit as result
	    do i = 1, n {
		fdata[i] = Memr[fit+i-1]
		}
	    }

# If plot enabled, show the object spectrum with continuum removed
	if (pltfit) {
	    call strcpy (name, title_line, SZ_LINE)
	    call strcat (" ", title_line, SZ_LINE)
	    call strcat (fitfunc, title_line, SZ_LINE)
	    call strcat (" fit", title_line, SZ_LINE)
	    ns = nsm
	    call tresplot (n,fdata,title_line,wltemp, wlab, ns, intfit, ymin, ymax)
	    }

	call ic_closer (ic)
	call cvfree (cv)
	call sfree (sp)
	return
end

# Fitting functions
define	FIT_TYPE	"|spline3|legendre|chebyshev|spline1|"
define	F_SPLINE3	1
define	F_LEGENDRE	2
define	F_CHEBYSHEV	3
define	F_SPLINE1	4


# RECOVER_ICFIT_PARS - Since the ICFIT parameters may have been changed in
# an interactive operation, we need to get the new values from the ICFIT
# structure.

procedure recover_icfit_pars (ic)

pointer	ic		# ICFIT pointer

int	strdic(), ic_geti()
real	ic_getr()

include "tresop.com"

begin
	naverage = ic_geti (ic, "naverage")
	fitorder = ic_geti (ic, "order")
	niterate = ic_geti (ic, "niterate")
	lowrej = ic_getr (ic, "low")
	hirej = ic_getr (ic, "high")
	grow = ic_getr (ic, "grow")

	call ic_gstr (ic, "sample", sample, SZ_LINE)
	call ic_gstr (ic, "function", fitfunc, SZ_LINE)
	function = strdic (fitfunc, fitfunc, SZ_LINE, FIT_TYPE)
	if (function == 0)
	    call error (0, "Unknown fitting function type")
	return
end

# GETFITPARS -- Get fitting parameters from the parameter file


procedure getfitpars ()

int	strdic(), clgeti()
real	clgetr()
bool	clgetb(), streq()

include "tresop.com"

begin

	choplines = clgetb ("choplines")
	emrej = clgetr ("em_reject")
	abrej = clgetr ("abs_reject")

	outfit = clgetb ("fit_res")
	intfit = clgetb ("fit_int")
	pltfit = clgetb ("fit_plot")
	fitint = clgetb("f_interact")
	call clgstr ("f_function", fitfunc, SZ_LINE)
	if (streq(fitfunc,"") || streq(fitfunc," "))
	    call error (0,"f_function specified as empty string.")
	function = strdic (fitfunc, fitfunc, SZ_LINE, FIT_TYPE)
	if (function == 0) 
	    call error (0, "Unknown fitting function type")

	call strcpy ("*", sample, SZ_FNAME)
	fitorder = clgeti ("f_order")
	niterate = clgeti ("f_niterate")
	naverage = clgeti ("f_naverage")
	grow = clgetr ("f_grow")
	lowrej = clgetr ("f_low_reject")
	hirej = clgetr ("f_high_reject")

end

# Feb 10 2009	New task based on rvsao.icsubcon() and rvsao/Util/contpars.x
