# File t_tmaskfile.x
# May 21 2009
# By Doug Mink

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXCOL		4096
define	MAXLINE		4700

# TMASKFILE -- Make a file to mask out spectra from flattening file

procedure t_tmaskfile ()

pointer	im1, im2, linein, lineout
real	datamask[MAXCOL,MAXLINE]
char	infile[SZ_FNAME]
char	maskfile[SZ_FNAME]
char	datedir[SZ_FNAME]
char	dstr[SZ_LINE]
char	hstr[SZ_LINE]
long	v[IM_MAXDIM]
bool	verbose, debug
int	i, j, nc, nl
real	datai
pointer	imgl2r(), immap()
int	impnlr()
bool	clgetb()
long	clktime()

begin

# Get parameters
	call clgstr ("infile", infile, SZ_FNAME)
	call clgstr ("maskfile", maskfile, SZ_FNAME)
	call clgstr ("datedir", datedir, SZ_FNAME)
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

# Open the input image
	iferr (im1 = immap (infile, READ_ONLY, 0)) {
	    call printf ("TMASKFILE: Cannot read %s\n")
		call pargstr (infile)
	    call clputi ("error", 1)
	    return
	    }
	nc = IM_LEN(im1,1)
	nl = IM_LEN(im1,2)
	if (verbose) {
	    call printf ("TMASKFILE: %d x %d input flattening image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (infile)
	    call flush (STDOUT)
	    }

# Read from the median image and process each pixel
	do j = 1, nl {

# Get input buffer for this line of the image
	    linein = imgl2r (im1, j)
	    if (linein == NULL) {
		call printf ("TMASKFILE: Cannot read line %d of %s\n")
		    call pargi (i)
		    call pargstr (infile)
		call flush (STDOUT)
		call clputi ("error", 3)
		call imunmap (im1)
		return
		}

# Zero all pixels which are not =1
	    do i = 1, nc {
		datai = Memr[linein+i-1]
		if (datai > 1.0) {
		    datai = 0.0
		    }
		else if (datai < 1.0) {
		    datai = 0.0
		    }
		datamask[i,j] = datai
		}
	    if (debug) {
		call printf ("TMASKFILE: Line %4d set.\n")
		    call pargi (j)
		call flush (STDOUT)
		}
	    }

# If any pixel =1 is not adjacent to another pixel =1, zero it
	do j = 1, nl {
	    do i = 1, nc {
		datai = datamask[i,j]
		if (datai > 0.0) {

# Pixel to the left
		    if (i > 1) {
			if (datamask[i-1,j] > 0.0)
			    next
			}

# Pixel to the right
		    if (i < nc) {
			if (datamask[i+1,j] > 0.0)
			    next
			}

# Pixel above left
		    if (j > 1) {
			if (i > 1) {
			    if (datamask[i-1,j-1] > 0.0)
				next
			    }

# Pixel above
			if (datamask[i,j-1] > 0.0)
			    next

# Pixel above right
			if (i > 1) {
			    if (datamask[i-1,j-1] > 0.0)
				next
			    }
			}

# Pixel to the left
		    if (i > 1) {
			if (datamask[i-1,j] > 0.0)
			    next
			}

# Pixel to the right
		    if (i < nc) {
			if (datamask[i+1,j] > 0.0)
			    next
			}

# Pixel above left
		    if (j < nl) {
			if (i > 1) {
			    if (datamask[i-1,j-1] > 0.0)
				next
			    }

# Pixel above
			if (datamask[i,j-1] > 0.0)
			    next

# Pixel above right
			if (i > 1) {
			    if (datamask[i-1,j-1] > 0.0)
				next
			    }
			}
		    }
		}
	    }
	if (verbose) {
	    call printf ("TMASKFILE: %d x %d mask image created\n")
		call pargi (nc)
		call pargi (nl)
	    call flush (STDOUT)
	    }

# Open the output mask file
	iferr (im2 = immap (maskfile, NEW_COPY, im1)) {
	    call printf ("TMASKFILE: Cannot create mask file %s\n")
		call pargstr (maskfile)
	    call flush (STDOUT)
	    call clputi ("error", 2)
	    return
	    }
	else if (debug) {
	    call printf ("TMASKFILE: Mask file %s created\n")
		call pargstr (maskfile)
	    call flush (STDOUT)
	    }

	call imastr (im2, "i_title", "TRES Mask image")
	nc = IM_LEN(im2,1)
	nl = IM_LEN(im2,2)
	if (verbose) {
	    call printf ("TMASKFILE: %d x %d mask image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (maskfile)
	    call flush (STDOUT)
	    }

# Set MASKFILE header keyword
	call logtime (dstr, SZ_LINE)
#	if (verbose) {
#	    call printf ("%s Mask file created from %s/%s\n")
#		call pargstr (dstr)
#		call pargstr (datedir)
#		call pargstr (infile)
#	    call flush (STDOUT)
#	    }
	call sprintf (hstr,SZ_LINE,"%s Mask file created from %s/%s")
	    call pargstr (dstr)
	    call pargstr (datedir)
	    call pargstr (infile)
	call imastr (im2, "MASKFILE", hstr)
	if (verbose) {
	    call printf ("TMASKFILE: %s\n")
		call pargstr (hstr)
	    call flush (STDOUT)
	    }
	IM_CTIME(im2) = clktime (long(0))
	IM_UPDATE(im2) = YES

# Initialize vector for output
	call amovkl (long(1), v, IM_MAXDIM)

# Write the output file one line at a time
	do j = 1, nl {

# Get output buffer for this line of the image
	    v[2] = j
	    if (impnlr (im2, lineout, v) < nc) {
		call printf ("TMASKFILE: Cannot write line %d of %s\n")
		    call pargi (j)
		    call pargstr (maskfile)
		    call flush (STDOUT)
		call clputi ("error", 3)
		return
		}
	    call achtrr (datamask[1,j], Memr[lineout], nc)
	    }
	if (verbose) {
	    call printf ("TMASKFILE: Mask image transferred to output buffer\n")
	    call flush (STDOUT)
	    }

# Close output image
	if (verbose) {
	    call printf ("TMASKFILE: Ready to write mask image %s\n")
		call pargstr (maskfile)
	    call flush (STDOUT)
	    }
	call imunmap (im2)
	if (verbose) {
	    call printf ("TMASKFILE: Output mask image %s written.\n")
		call pargstr (maskfile)
	    call flush (STDOUT)
	    }

# Close input image
	call imunmap (im1)
	if (verbose) {
	    call printf ("TMASKFILE: Input image %s closed.\n")
		call pargstr (infile)
	    call flush (STDOUT)
	    }

end
# Apr 20 2009	New program based on tres.tcoslim
# May 21 2009	Drop unused declaration
